# Generated by Django 2.0.6 on 2018-11-19 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("vng_api_common", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="APICredential",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("api_root", models.URLField(unique=True, verbose_name="api root")),
                (
                    "client_id",
                    models.CharField(max_length=255, verbose_name="client id"),
                ),
                ("secret", models.CharField(max_length=255, verbose_name="secret")),
            ],
            options={
                "verbose_name": "API credential",
                "verbose_name_plural": "API credentials",
            },
        )
    ]
