# Generated by Django 2.0.13 on 2019-04-29 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("notifications", "0006_auto_20190417_1142")]

    operations = [
        migrations.AlterField(
            model_name="notificationsconfig",
            name="api_root",
            field=models.URLField(unique=True, verbose_name="api root"),
        )
    ]
