# Generated by Django 2.1.1 on 2019-03-19 09:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("notifications", "0002_subscription__subscription")]

    operations = [
        migrations.RenameField(
            model_name="notificationsconfig", old_name="location", new_name="api_root"
        ),
        migrations.AddField(
            model_name="notificationsconfig",
            name="client_id",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="client id"
            ),
        ),
        migrations.AddField(
            model_name="notificationsconfig",
            name="secret",
            field=models.CharField(blank=True, max_length=255, verbose_name="secret"),
        ),
        migrations.AlterField(
            model_name="notificationsconfig",
            name="api_root",
            field=models.URLField(unique=True, verbose_name="api root"),
        ),
    ]
