# Generated by Django 2.0.13 on 2019-05-03 09:41

from django.db import migrations, models

import vng_api_common.fields


class Migration(migrations.Migration):

    dependencies = [("authorizations", "0003_auto_20190502_0409")]

    operations = [
        migrations.AlterField(
            model_name="autorisatie",
            name="max_vertrouwelijkheidaanduiding",
            field=vng_api_common.fields.VertrouwelijkheidsAanduidingField(
                blank=True,
                choices=[
                    ("openbaar", "OPENBAAR"),
                    ("beperkt openbaar", "BEPERKT OPENBAAR"),
                    ("intern", "INTERN"),
                    ("zaakvertrouwelijk", "ZAAKVERTROUWELIJK"),
                    ("vertrouwelijk", "VERTROUWELIJK"),
                    ("confidentieel", "CONFIDENTIEEL"),
                    ("geheim", "GEHEIM"),
                    ("zeer geheim", "ZEER GEHEIM"),
                ],
                help_text="Maximum level of confidentiality that is allowed",
                max_length=20,
            ),
        ),
        migrations.AlterField(
            model_name="autorisatie",
            name="zaaktype",
            field=models.URLField(
                blank=True,
                help_text="Url of the zaaktype that is allowed",
                max_length=1000,
            ),
        ),
    ]
