# Generated by Django 2.1.1 on 2019-07-12 15:41

from django.db import migrations, models

import vng_api_common.fields


class Migration(migrations.Migration):

    dependencies = [("authorizations", "0007_auto_20190506_1212")]

    operations = [
        migrations.AlterField(
            model_name="authorizationsconfig",
            name="component",
            field=models.CharField(
                choices=[
                    ("ac", "Autorisatiecomponent"),
                    ("nrc", "Notificatierouteringcomponent"),
                    ("zrc", "Zaakregistratiecomponent"),
                    ("ztc", "Zaaktypecatalogus"),
                    ("drc", "Documentregistratiecomponent"),
                    ("brc", "Besluitregistratiecomponent"),
                ],
                default="zrc",
                max_length=50,
                verbose_name="component",
            ),
        ),
        migrations.AlterField(
            model_name="autorisatie",
            name="component",
            field=models.CharField(
                choices=[
                    ("ac", "Autorisatiecomponent"),
                    ("nrc", "Notificatierouteringcomponent"),
                    ("zrc", "Zaakregistratiecomponent"),
                    ("ztc", "Zaaktypecatalogus"),
                    ("drc", "Documentregistratiecomponent"),
                    ("brc", "Besluitregistratiecomponent"),
                ],
                help_text="Component waarop autorisatie van toepassing is.",
                max_length=50,
                verbose_name="component",
            ),
        ),
        migrations.AlterField(
            model_name="autorisatie",
            name="max_vertrouwelijkheidaanduiding",
            field=vng_api_common.fields.VertrouwelijkheidsAanduidingField(
                blank=True,
                choices=[
                    ("openbaar", "Openbaar"),
                    ("beperkt_openbaar", "Beperkt openbaar"),
                    ("intern", "Intern"),
                    ("zaakvertrouwelijk", "Zaakvertrouwelijk"),
                    ("vertrouwelijk", "Vertrouwelijk"),
                    ("confidentieel", "Confidentieel"),
                    ("geheim", "Geheim"),
                    ("zeer_geheim", "Zeer geheim"),
                ],
                help_text="Maximaal toegelaten vertrouwelijkheidaanduiding (inclusief).",
                max_length=20,
            ),
        ),
    ]
