a:link,a:visited{color:#447e9b}a:focus,a:hover{color:#036}#header{color:#ffc;background:#417690;position:-webkit-sticky;position:sticky;top:0;z-index:100000}#header a:link,#header a:visited{color:#fff}#user-tools a{border-bottom:0;text-decoration:underline}#user-tools a:focus,#user-tools a:hover{color:#79aec8}.inline-group h2,.module caption,.module h2{background:#79aec8}#branding h1{color:#f5dd5d}#branding h1 a:link,#branding h1 a:visited{color:#f5dd5d}div.breadcrumbs{background:#79aec8;color:#c4dce8;position:-webkit-sticky;position:sticky;top:84px !important;z-index:100000}div.breadcrumbs a,div.breadcrumbs a:focus,div.breadcrumbs a:hover{color:#fff}#changelist-filter a:focus,#changelist-filter a:hover,#changelist-filter li.selected a{color:#447e9b!important}#changelist-filter li.selected a:focus,#changelist-filter li.selected a:hover{color:#79aec8!important}.object-tools a:focus,.object-tools a:hover{background-color:#333}.button,.submit-row input,a.button,input[type=button],input[type=submit]{background:#79aec8;color:#fff}.button:active,.button:focus,.button:hover,input[type=button]:active,input[type=button]:focus,input[type=button]:hover,input[type=submit]:active,input[type=submit]:focus,input[type=submit]:hover{background:#417690}.button.default,.submit-row input.default,input[type=submit].default{background:#417690}.button.default:active,.button.default:focus,.button.default:hover,input[type=submit].default:active,input[type=submit].default:focus,input[type=submit].default:hover{background:#000}.delete-confirmation form .cancel-link{background:#c4dce8;color:#fff}.delete-confirmation form .cancel-link:active,.delete-confirmation form .cancel-link:focus,.delete-confirmation form .cancel-link:hover{background:#333}
