# Generated by Django 2.2.6 on 2020-04-16 04:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("testapp", "0002_auto_20190620_0849"),
    ]

    operations = [
        migrations.AddField(
            model_name="group",
            name="subgroup_field_1",
            field=models.CharField(blank=True, max_length=50),
        ),
        migrations.AddField(
            model_name="group",
            name="subgroup_field_2",
            field=models.CharField(blank=True, default="baz", max_length=50),
        ),
    ]
