# Generated by Django 2.1.8 on 2019-05-06 12:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("authorizations", "0006_auto_20190506_0901")]

    operations = [
        migrations.AddField(
            model_name="autorisatie",
            name="besluittype",
            field=models.URLField(
                blank=True,
                help_text="URL naar het besluittype waarop de autorisatie van toepassing is.",
                max_length=1000,
                verbose_name="besluittype",
            ),
        ),
        migrations.AddField(
            model_name="autorisatie",
            name="informatieobjecttype",
            field=models.URLField(
                blank=True,
                help_text="URL naar het informatieobjecttype waarop de autorisatie van toepassing is.",
                max_length=1000,
                verbose_name="informatieobjecttype",
            ),
        ),
    ]
