# Generated by Django 2.2.4 on 2019-09-18 13:35

import django.contrib.postgres.indexes
from django.contrib.postgres.operations import TrigramExtension
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("audittrails", "0010_audittrail_request_id")]

    operations = [
        TrigramExtension(),
        migrations.AddIndex(
            model_name="audittrail",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["hoofd_object"],
                name="audittrail_hoofdobject_trgm",
                opclasses=["gin_trgm_ops"],
            ),
        ),
    ]
